-- 	Copyright  2009.	GSI Technology
-- 						Jeff Daugherty
-- 						apps@gsitechnology.com
--  Version: 3.9
--
-- 	FileName: core.vhd
-- 	Unified Sram Core Model for Sync Burst/NBT Sram
--
-- 	Revision History:
--   04/23/02  1.0  1) Created VHDL Core.VHD from Verilog Core.V
--   06/05/02  1.1  1) added new signals, DELAY and tKQX.  These signals will
--                     be used to setup the Clock to Data Invalid  spec.
--   07/17/02  1.2  1) Fixed the JTAG State machine
--                  2) changed the SR register to shift out the MSB and shift
--                     in the LSB
--   09/25/02  1.3  1) Removed all nPE pin features
--                  2) Max number of Core addresses is now dynamic
--                  3) Max width of Core data is now dynamic
--                  4) Removed alll reference of JTAG from core, seperate JTAG
--                     model file: GSI_JTAG
--   01/10/03  1.4  1) Created a Write_Array process to remove race conditions
--                  2) Created a Read_Array proccess to remove race conditions
--   02/20/03  1.5  1) Added We and Waddr to Read_Array sensitivity list.
--                  2) Changed the Read_Array process to look at the last
--                     write's byte write setting and determine where to pull the
--                     read data from, either coherency(byte write on) or the
--                     array(byte write off).
--                  3) Added signal Iscd to fix SCD to the right state for NBT
-- 04/03/03   1.6   1) Added a write clock W_k to trigger the Write_Array function
-- 07/09/03   1.7   1) changed NBT write clock to clock off of we2.
--                  2) Delayed the internal clock by 1ns to control the write
--                  3) changed ce to take into account NBT mode
-- 07/23/03   1.8   1) Changed W_K to ignore the byte writes
-- 08/12/03   1.9   1) updated state machine to include seperate read and write
--                     burst states
--                  2) Changed internal bytewrite signal to ignore nW
-- 10/29/03   2.0   1) updated the state machine, changed reference to suspend
--                     to deselect. 
--                  2) added timing functions to core
-- 03/25/04   2.1   1) Updated state machine.  Added deselect and suspend states
--                  2) Fixed other issues with the state machine
-- 04/28/04   2.2   1) Rearranged state that determins Deselect, Burst and Suspend
--
-- 12/01/05   3.0   1) Created an NBT only model
-- 02/07/05   3.1   1) Added checker to check for 100ps or less resolution
--                  2) CLK_i is now 100ps from CK
-- 06/21/06   3.2   1) Added Qswitch to control when the IOs turn on or off
--                  2) Delayed the Qxi inteernal data busses instead of the DQx
--                  external Data busses.
--                  3) Added CLK_i2 to control the setting of Qswitch
--                  4) All these changes removed Negative time issue for some simulations
--07/18/06    3.3   1) Initialized ce and re to 0 so that Qswitch is not
--                     undfined which can cause bus contention on startup.
-- 09/27/06   3.4   1) Fixed state machine for E=1 and ADV=1 not bursting
--
-- 09/30/06   3.5   1) Added bw0 <= BW; to state NewCycle when BurstWR is loaded
-- 05/15/07   3.6   1) updated QSWITCH to include nG as an input
-- 02/04/09   3.7   1) Removed GSI library requirement
-- 07/30/09   3.8   1) Model now has a reduced memory core.  The size is controlled by A_BITS
-- 09/29/09   3.9   1) Fixed Read_array not using address_index, was using raddr.
--
--
LIBRARY ieee;
USE ieee.std_logic_1164.all;
ENTITY VHDL_NBT_CORE IS
  GENERIC (
    CONSTANT bank_size   : integer;
    CONSTANT A_size      : integer;
    CONSTANT A_bits      : integer;
    CONSTANT DQ_size     : integer);
  PORT (
    SIGNAL A     : IN std_logic_vector(A_size - 1 DOWNTO 0);-- address
    SIGNAL DQa   : INOUT std_logic_vector(DQ_size DOWNTO 1) ;-- byte A data
    SIGNAL DQb   : INOUT std_logic_vector(DQ_size DOWNTO 1) ;-- byte B data
    SIGNAL DQc   : INOUT std_logic_vector(DQ_size DOWNTO 1) ;-- byte C data
    SIGNAL DQd   : INOUT std_logic_vector(DQ_size DOWNTO 1) ;-- byte D data
    SIGNAL DQe   : inout std_logic_vector(DQ_size DOWNTO 1) ;-- byte E data
    SIGNAL DQf   : inout std_logic_vector(DQ_size DOWNTO 1) ;-- byte F data
    SIGNAL DQg   : inout std_logic_vector(DQ_size DOWNTO 1) ;-- byte G data
    SIGNAL DQh   : inout std_logic_vector(DQ_size DOWNTO 1) ;-- byte H data
    SIGNAL nBa   : IN std_logic;-- bank A write enable
    SIGNAL nBb   : IN std_logic;-- bank B write enable
    SIGNAL nBc   : IN std_logic;-- bank C write enable
    SIGNAL nBd   : IN std_logic;-- bank D write enable
    SIGNAL nBe   : IN std_logic;-- bank E write enable
    SIGNAL nBf   : IN std_logic;-- bank F write enable
    SIGNAL nBg   : IN std_logic;-- bank G write enable
    SIGNAL nBh   : IN std_logic;-- bank H write enable
    SIGNAL CK    : IN std_logic;-- clock
    SIGNAL nCKE  : IN std_logic;-- only for NBT devices
    SIGNAL nW    : IN std_logic;-- NBT write strobe
    SIGNAL nE1   : IN std_logic;-- chip enable 1
    SIGNAL E2    : IN std_logic;-- chip enable 2
    SIGNAL nE3   : IN std_logic;-- chip enable 3
    SIGNAL nG    : IN std_logic;-- output enable
    SIGNAL pADV  : IN std_logic;-- Advance not / load -> NBT Only
    SIGNAL ZZ    : IN std_logic;-- power down
    SIGNAL nFT   : IN std_logic;-- Pipeline / Flow through
    SIGNAL nLBO  : IN std_logic;-- Linear Burst Order not
    SIGNAL HighZ : std_logic_vector(DQ_size downto 1) ;
    SIGNAL tKQ   : time;
    SIGNAL tKQX  : time);
END VHDL_NBT_CORE;

library ieee;
LIBRARY Std;
ARCHITECTURE GSI_NBT_CORE OF VHDL_NBT_CORE IS
  USE Std.textio.ALL;
  use ieee.std_logic_unsigned.all, ieee.std_logic_arith.all;
  TYPE MEMORY  IS ARRAY (0 TO bank_size) OF std_logic_vector((8*DQ_size)-1 DOWNTO 0);
  TYPE AMEMORY IS ARRAY (0 TO bank_size) OF std_logic_vector(A_size-1 DOWNTO 0);
--   ******** Define Sram Operation Mode    **********************
  shared variable memory_core  : MEMORY ;
  shared variable address_core : AMEMORY ;
-- ---------------------------------------------------------------
--   	Merging Input Pins of NBT / Burst Operation
-- ---------------------------------------------------------------
  SIGNAL CKE  : std_logic;-- internal clock enable
  SIGNAL E    : std_logic;-- internal chip enable
  SIGNAL BW   : std_logic_vector(7 DOWNTO 0);-- internal byte write enable
  SIGNAL ADV  : std_logic;-- internal address advance
  SIGNAL ZNOP : std_logic;-- NBT mode the nop cycle
  SIGNAL W    : std_logic;
  SIGNAL R    : std_logic;
  SIGNAL W_k    : std_logic;
  SIGNAL R_k    : std_logic;
-- ---------------------------------------------------------------
--   	Gated SRAM Clock
-- ---------------------------------------------------------------
  SIGNAL clk_i : std_logic := '0';
  SIGNAL clk_i2 : std_logic := '0';
-- ---------------------------------------------------------------
--   	reduced memory stuff
-- ---------------------------------------------------------------
  signal address_used : std_logic_vector(a_bits-1 downto 0) := (others=>'0');
  constant MAX_BITS   : std_logic_vector(a_bits-1 downto 0) := (others=>'1');
-- ---------------------------------------------------------------
--   	Combinatorial Logic
-- ---------------------------------------------------------------
--  Internal data is always 9 bits
  SIGNAL Qxi   : std_logic_vector(8*DQ_size-1 DOWNTO 0);-- read data
  SIGNAL Dxi   : std_logic_vector(8*DQ_size-1 DOWNTO 0);-- write data
  SIGNAL bw0   : std_logic_vector(7 DOWNTO 0)  := conv_std_logic_vector(0, 8);-- saved bw
  SIGNAL bw1   : std_logic_vector(7 DOWNTO 0)  := conv_std_logic_vector(0, 8);-- bw buffer 1
  SIGNAL bw2   : std_logic_vector(7 DOWNTO 0)  := conv_std_logic_vector(0, 8);-- bw buffer 2
  SIGNAL addr0 : std_logic_vector(A_size - 1 DOWNTO 0):= conv_std_logic_vector(0,A_size);-- saved address
  SIGNAL addr1 : std_logic_vector(A_size - 1 DOWNTO 0):= conv_std_logic_vector(0,A_size);-- address buffer 1
  SIGNAL addr2 : std_logic_vector(A_size - 1 DOWNTO 0):= conv_std_logic_vector(0,A_size);-- address buffer 2
  SIGNAL baddr : std_logic_vector(A_size - 1 DOWNTO 0):= conv_std_logic_vector(0,A_size);-- burst memory address
  SIGNAL waddr : std_logic_vector(A_size - 1 DOWNTO 0):= conv_std_logic_vector(0,A_size);-- write memory address
  SIGNAL raddr : std_logic_vector(A_size - 1 DOWNTO 0):= conv_std_logic_vector(0,A_size);-- read memory address
  SIGNAL bcnt  : std_logic_vector(1 DOWNTO 0)  := conv_std_logic_vector(0, 2);-- burst counter
  SIGNAL we0   : std_logic  := '0';
  SIGNAL we1   : std_logic  := '0';
  SIGNAL we2   : std_logic  := '0';-- we flags
  SIGNAL re0   : std_logic  := '0';
  SIGNAL re1   : std_logic  := '0';
  SIGNAL re2   : std_logic  := '0';-- re flags
  SIGNAL ce0   : std_logic  := '0';
  SIGNAL ce1   : std_logic  := '0';
  SIGNAL ce2   : std_logic  := '0';-- ce flags
  SIGNAL ce    : std_logic  := '0';
  SIGNAL re    : std_logic  := '0';-- active ones
  SIGNAL oe    : std_logic  := '0';
  SIGNAL we    : std_logic  := '0';
  SIGNAL Qswitch: std_logic;-- active ones
  SIGNAL bwi   : std_logic_vector(7 DOWNTO 0);-- active ones
  SIGNAL state : string (8 DOWNTO 1) := "DESELECT";

  SIGNAL Check_Time : time := 1.0 ns;
  SIGNAL DELAY      : time := 1.0 ns;
  SIGNAL OFFSET     : time   := 0 ps;
  SIGNAL GUARD      : boolean:= TRUE;

-- ---------------------------------------------------------------
--   	State Machine
-- ---------------------------------------------------------------
  function POSEDGE (SIGNAL s : std_ulogic) return BOOLEAN IS
  begin
    RETURN (s'EVENT AND ((To_X01(s'LAST_VALUE) = '0') OR (s = '1')));
  end; 
  function NEGEDGE (SIGNAL s : std_ulogic) return BOOLEAN IS
  begin
    RETURN (s'EVENT AND ((To_X01(s'LAST_VALUE) = '1') OR (s = '0')) );
  end; 
-- END TIMING FUNCTIONS
  
  PROCEDURE shiftnow (SIGNAL addr2 : INOUT std_logic_vector(A_size - 1 DOWNTO 0) ;
                      SIGNAL addr1 : INOUT std_logic_vector(A_size - 1 DOWNTO 0) ;
                      SIGNAL bw2   : INOUT std_logic_vector(7 DOWNTO 0) ;
                      SIGNAL bw1   : INOUT std_logic_vector(7 DOWNTO 0) ;
                      SIGNAL we2   : INOUT std_logic ;
                      SIGNAL we1   : INOUT std_logic ;
                      SIGNAL re2   : INOUT std_logic ;
                      SIGNAL re1   : INOUT std_logic ;
                      SIGNAL ce2   : INOUT std_logic ;
                      SIGNAL ce1   : INOUT std_logic ;
                      SIGNAL Dxi   : INOUT std_logic_vector(71 DOWNTO 0))
  IS
  BEGIN
    addr2 <= addr1;
    addr1 <= baddr;
    bw2 <= bw1;
    bw1 <= bw0;
    we2 <= we1;
    we1 <= we0;
    re2 <= re1;
    re1 <= re0;
    ce2 <= ce1;
    ce1 <= ce0;
-- save data bus for later write
    Dxi <= (DQh & DQg & DQf & DQe & DQd & DQc & DQb & DQa);
  END;

BEGIN
  PROCESS
  BEGIN
    WAIT UNTIL POSEDGE(CK);
    clk_i <= (CKE AND NOT ZZ) after OFFSET;
    clk_i2 <= (CKE AND NOT ZZ) after (OFFSET + 100 ps);
    WAIT UNTIL NEGEDGE(CK);
    clk_i <= '0' after OFFSET;
    clk_i2 <= '0' after (OFFSET + 100 ps);
  END PROCESS;

  st : PROCESS
    variable tstate : string(8 DOWNTO 1) :="DESELECT";
    variable twe0 : std_logic := '0';
    variable tre0 : std_logic := '0';
    variable tce0 : std_logic := '0';
  BEGIN
    WAIT UNTIL POSEDGE(CLK_i);
    CASE state IS
      WHEN "DESELECT" =>
        if E = '1' and ADV = '0'  then
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          twe0 := W;
          tre0 := R;
          tce0 := E;
          addr0 <= A;
          bw0 <= BW;
          bcnt <= "00";
          tstate := "NEWCYCLE";
        END IF;
        if (E = '0' or ADV = '1') then
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "DESELECT";
          twe0 := '0';
          tre0 := '0';
          tce0 := '0';
        END IF;
-- **************************************************
      WHEN "NEWCYCLE" =>
        --enables active, ADV disable and either ADSP or ADSP active
        IF (E = '1' AND ADV = '0') THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "NEWCYCLE";
          twe0 := W;
          tre0 := R;
          tce0 := E;
          addr0 <= A;
          bw0 <= BW;
          bcnt <= "00";
        END IF;
        -- enables, ADSP, ADSC and ADV disabled
        IF (E = '0') then
          if (ADV = '0') then
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
            tstate := "DESELECT";
            twe0 := '0';
            tre0 := '0';
            tce0 := '0';
          end if;
        end if;
--  Burst Start
        IF ADV = '1' THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          if we0 = '1' then
            tstate := "BURSTWR ";
            bw0 <= BW;
--            twe0 := (ADV AND NOT ZNOP);
            twe0 := ADV;
            tre0 := '0';
          end if;
          if re0 = '1' then
            tstate := "BURSTRD ";
            twe0 := '0';
--            tre0 := (ADV AND NOT ZNOP);
            tre0 := (ADV AND NOT W);
          end if;
          tce0 := '1';
          bcnt <= bcnt + "01";
        end if;
--- **************************************************
      WHEN "BURSTRD " =>
        IF (E = '0' and ADV = '0') THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "DESELECT";
          twe0 := '0';
          tre0 := '0';
          tce0 := '0';
        END IF;
        IF (E = '1' and ADV = '0') THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "NEWCYCLE";
          twe0 := W;
--          tre0 := (R AND NOT ZNOP);
          tre0 := (R AND NOT W);
          tce0 := '1';
          addr0 <= A;
          bw0 <= BW;
          bcnt <= "00";
        END IF;
        IF (ADV = '1') THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "BURSTRD ";
--          tre0 := (ADV AND NOT ZNOP);
          tre0 := (ADV AND NOT W);
          tce0 := '1';
          bw0 <= BW;
          bcnt <= bcnt + "01";
        END IF;
-- **************************************************
      WHEN "BURSTWR " =>
        IF (E = '0' and ADV = '0') THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "DESELECT";
          twe0 := '0';
          tre0 := '0';
          tce0 := '0';
        END IF;
        IF (E = '1' and ADV = '0') THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "NEWCYCLE";
          twe0 := W;
--          tre0 := (R AND NOT ZNOP);
          tre0 := (R AND NOT W);
          tce0 := '1';
          addr0 <= A;
          bw0 <= BW;
          bcnt <= "00";
        END IF;
        IF (ADV = '1') THEN
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
          tstate := "BURSTWR ";
--          twe0 := (ADV AND NOT ZNOP);
          twe0 := ADV;
          tce0 := '1';
          bw0 <= BW;
          bcnt <= bcnt + "01";
        END IF;
-- **************************************************
      WHEN OTHERS =>
          shiftnow(addr2, addr1, bw2, bw1, we2, we1, re2, re1, ce2, ce1, Dxi);
        tstate := "DESELECT";
        twe0 := '0';
        tre0 := '0';
        tce0 := '0';
        bcnt <= "00";
    END CASE;
    state <= tstate;
    we0 <= twe0;
    re0 <= tre0;
    ce0 <= tce0;
  END PROCESS;
-- ---------------------------------------------------------------
--               Data IO Logic
-- ---------------------------------------------------------------
  Write_Array: process (W_k)
    variable addr_t         : integer;
    variable index          : integer := 0;
    variable address_match  : integer := 0;
    variable address_index  : integer := 0;
  begin  -- process Write_Array
    IF (POSEDGE(W_k) and we='1') THEN
      if conv_integer(address_used) >0 then
        for index in 0 to conv_integer(address_used)-1 loop
          if (address_core(index) = waddr and address_match=0) then
            address_match := 1;
            address_index := index;
          end if;
        end loop;  -- index
      end if;
      if address_match = 1 then
        address_core(address_index) := waddr;
        addr_t := address_index;
        address_match := 0;
      else
        if address_used = MAX_BITS then
          report "A Memory overflow has occurred.  Increase Size of A_BITS to increase memory size" severity failure;
        end if;
        address_core(conv_integer(address_used)) := waddr;
        addr_t := conv_integer(address_used);
        address_used <= address_used + 1;
      end if;
      IF bwi(0) = '1' THEN
        memory_core(addr_t)((DQ_size-1) downto 0)             := Dxi((DQ_size-1) downto 0);
      END IF;
      IF bwi(1) = '1' THEN
        memory_core(addr_t)(2*(DQ_size)-1 downto DQ_size)     := Dxi(2*(DQ_size)-1 downto (DQ_size));
      END IF;
      IF bwi(2) = '1' THEN
        memory_core(addr_t)(3*(DQ_size)-1 downto 2*(DQ_size)) := Dxi(3*(DQ_size)-1 downto 2*(DQ_size));
      END IF;
      IF bwi(3) = '1' THEN
        memory_core(addr_t)(4*(DQ_size)-1 downto 3*(DQ_size)) := Dxi(4*(DQ_size)-1 downto 3*(DQ_size));
      END IF;
      IF bwi(4) = '1' THEN
        memory_core(addr_t)(5*(DQ_size)-1 downto 4*(DQ_size)) := Dxi(5*(DQ_size)-1 downto 4*(DQ_size));
      END IF;
      IF bwi(5) = '1' THEN
        memory_core(addr_t)(6*(DQ_size)-1 downto 5*(DQ_size)) := Dxi(6*(DQ_size)-1 downto 5*(DQ_size));
      END IF;
      IF bwi(6) = '1'  THEN
        memory_core(addr_t)(7*(DQ_size)-1 downto 6*(DQ_size)) := Dxi(7*(DQ_size)-1 downto 6*(DQ_size));
      END IF;
      IF bwi(7) = '1'  THEN
        memory_core(addr_t)(8*(DQ_size)-1 downto 7*(DQ_size)) := Dxi(8*(DQ_size)-1 downto 7*(DQ_size));
      END IF;
    END IF;
  end process Write_Array;

  Read_Array: process (r_k)
    variable index          : integer := 0;
    variable address_match  : integer := 0;
    variable address_index  : integer := 0;
  begin  -- process Read_Array
    if conv_integer(address_used) > 0 then
      for index in 0 to conv_integer(address_used) - 1 loop
        if address_match = 0 then
          if address_core(index) = raddr then
            address_match := 1;
            address_index := index;
          end if;
        end if;
      end loop;  -- indev
    end if;
    if address_match = 1 then
      address_match := 0;
      IF (we = '0' OR raddr /= waddr) THEN
        Qxi <= transport memory_core(address_index) after DELAY - 200 ps;
      ELSE
        if bwi(0) = '1' then
          Qxi(DQ_size-1 downto 0) <= transport Dxi(DQ_size-1 downto 0) after DELAY - 200 ps;
        else
          Qxi(DQ_size-1 downto 0) <= transport memory_core(address_index)(DQ_size-1 downto 0) after DELAY - 200 ps;
        end if;
        if bwi(1) = '1' then
          Qxi(2*(DQ_size)-1 downto DQ_size) <= transport Dxi(2*(DQ_size)-1 downto DQ_size) after DELAY - 200 ps;
        else
          Qxi(2*(DQ_size)-1 downto DQ_size) <= transport memory_core(address_index)(2*(DQ_size)-1 downto DQ_size) after DELAY - 200 ps;
        end if;
        if bwi(2) = '1' then
          Qxi(3*(DQ_size)-1 downto 2*(DQ_size)) <= transport Dxi(3*(DQ_size)-1 downto 2*(DQ_size)) after DELAY - 200 ps;
        else
          Qxi(3*(DQ_size)-1 downto 2*(DQ_size)) <= transport memory_core(address_index)(3*(DQ_size)-1 downto 2*(DQ_size)) after DELAY - 200 ps;
        end if;
        if bwi(3) = '1' then
          Qxi(4*(DQ_size)-1 downto 3*(DQ_size)) <= transport Dxi(4*(DQ_size)-1 downto 3*(DQ_size)) after DELAY - 200 ps;
        else
          Qxi(4*(DQ_size)-1 downto 3*(DQ_size)) <= transport memory_core(address_index)(4*(DQ_size)-1 downto 3*(DQ_size)) after DELAY - 200 ps;
        end if;
        if bwi(4) = '1' then
          Qxi(5*(DQ_size)-1 downto 4*(DQ_size)) <= transport Dxi(5*(DQ_size)-1 downto 4*(DQ_size)) after DELAY - 200 ps;
        else
          Qxi(5*(DQ_size)-1 downto 4*(DQ_size)) <= transport memory_core(address_index)(5*(DQ_size)-1 downto 4*(DQ_size)) after DELAY - 200 ps;
        end if;
        if bwi(5) = '1' then
          Qxi(6*(DQ_size)-1 downto 5*(DQ_size)) <= transport Dxi(6*(DQ_size)-1 downto 5*(DQ_size)) after DELAY - 200 ps;
        else
          Qxi(6*(DQ_size)-1 downto 5*(DQ_size)) <= transport memory_core(address_index)(6*(DQ_size)-1 downto 5*(DQ_size)) after DELAY - 200 ps;
        end if;
        if bwi(6) = '1' then
          Qxi(7*(DQ_size)-1 downto 6*(DQ_size)) <= transport Dxi(7*(DQ_size)-1 downto 6*(DQ_size)) after DELAY - 200 ps;
        else
          Qxi(7*(DQ_size)-1 downto 6*(DQ_size)) <= transport memory_core(address_index)(7*(DQ_size)-1 downto 6*(DQ_size)) after DELAY - 200 ps;
        end if;
        if bwi(7) = '1' then
          Qxi(8*(DQ_size)-1 downto 7*(DQ_size)) <= transport Dxi(8*(DQ_size)-1 downto 7*(DQ_size)) after DELAY - 200 ps;
        else
          Qxi(8*(DQ_size)-1 downto 7*(DQ_size)) <= transport memory_core(address_index)(8*(DQ_size)-1 downto 7*(DQ_size)) after DELAY - 200 ps;
        end if;
      END IF;
    end if;
  end process Read_Array;

-- check it -t option is active and set correctly
  time_ck : process (CLK_i2)
  begin                                 --read clock controls outputs
      check_time <= abs(CK'last_event);
      assert check_time /= 0ns report "Resolution needs to be set to 100ps.  For modelSIM use vsim -t 100ps <>" severity FAILURE;
  end process time_ck;

  q_switch : process (CLK_i2)
  begin                                 --read clock controls outputs
  Qswitch <= transport (not nG and(re and ce)) after DELAY - 200 ps;
  end process q_switch;

  CKE  <= NOT nCKE;
  ADV  <= pADV;
  BW(0) <= NOT nBa;
  BW(1) <= NOT nBb;
  BW(2) <= NOT nBc;
  BW(3) <= NOT nBd;
  BW(4) <= NOT nBe;
  BW(5) <= NOT nBf;
  BW(6) <= NOT nBg;
  BW(7) <= NOT nBh;

  E    <= (NOT nE1 AND E2 AND NOT nE3);
  W    <=  NOT nW AND ( not nBa OR not nBb OR not nBc OR not nBd OR not nBe OR not nBf OR not nBg OR not nBh);
  R    <=  nW;
  W_k  <= we2 and clk_i after 1 ns when nFT='1' else we1 and clk_i after 1 ns;
  R_k  <= re1 and clk_i after 100 ps when nFT='1' else re0 and clk_i after 100 ps;
  ZNOP <=  nG OR not ((we and re) or oe) OR ZZ;

  baddr <= (addr0(A_size - 1 DOWNTO 2)&(bcnt(1) XOR addr0(1))&(bcnt(0) XOR addr0(0))) when nLBO='1'
           else (addr0(A_size - 1 DOWNTO 2)&(addr0(1 DOWNTO 0)+bcnt));
  waddr <= addr2 when nFT='1' else addr1;
  raddr <= addr1 when nFT='1' else baddr;
  bwi   <= bw2 when nFT='1' else bw1;
  we    <= we2 when nFT='1' else we1;
  re    <= re1 when nFT='1' else re0;
  ce    <= ce1 when nFT='1' else ce0;
  oe    <= re AND ce;

  DELAY   <= tKQX when ZNOP='1' else tKQ;
  
DQa   <= GUARDED Qxi((DQ_size)-1 downto 0)   when Qswitch = '1' else (others=>'Z');
DQb   <= GUARDED Qxi(2*(DQ_size)-1 downto (DQ_size))  when Qswitch = '1' else (others=>'Z');
DQc   <= GUARDED Qxi(3*(DQ_size)-1 downto 2*(DQ_size)) when Qswitch = '1' else (others=>'Z');
DQd   <= GUARDED Qxi(4*(DQ_size)-1 downto 3*(DQ_size)) when Qswitch = '1' else (others=>'Z');
DQe   <= GUARDED Qxi(5*(DQ_size)-1 downto 4*(DQ_size)) when Qswitch = '1' else (others=>'Z');
DQf   <= GUARDED Qxi(6*(DQ_size)-1 downto 5*(DQ_size)) when Qswitch = '1' else (others=>'Z');
DQg   <= GUARDED Qxi(7*(DQ_size)-1 downto 6*(DQ_size)) when Qswitch = '1' else (others=>'Z');
DQh   <= GUARDED Qxi(8*(DQ_size)-1 downto 7*(DQ_size)) when Qswitch = '1' else (others=>'Z');

END GSI_NBT_CORE;

